import { Binding } from '@microsoft/fast-element';
import { CaptureType } from '@microsoft/fast-element';
import { ComposableStyles } from '@microsoft/fast-element';
import { Constructable } from '@microsoft/fast-element';
import { Controller } from '@microsoft/fast-element';
import { CSSDesignToken } from '@microsoft/fast-foundation';
import { DesignSystem } from '@microsoft/fast-foundation';
import { ElementStyles } from '@microsoft/fast-element';
import { FASTElement } from '@microsoft/fast-element';
import { InterfaceSymbol } from '@microsoft/fast-foundation';
import { Logger as Logger_2 } from '@genesislcap/foundation-logger';
import { LoggerOptions as LoggerOptions_2 } from '@genesislcap/foundation-logger';
import { LogLevel } from '@genesislcap/foundation-logger';
import { SyntheticViewTemplate } from '@microsoft/fast-element';

/**
 * A design token that represents the active color scheme (light or dark).
 * @public
 */
export declare const activeColorScheme: CSSDesignToken<string>;

/**
 * The Genesis Server URL (WebSocket or HTTP).
 * @remarks Optional.
 * @defaultValue `${PROTOCOL}//${location.host}/${_SOCKET_EXT}/`
 * @example ws://localhost/gwf/, ws://localhost:9064
 * @public
 */
export declare let API_HOST: string;

/**
 * assureDesignSystem.
 *
 * @remarks
 * Webpack caching strategies can backfill missing modules with placeholders in an attempt to prevent errors. In the case
 * of offline remotes, we rely on the error to fall back to local versions. This utility assures we have a design system.
 *
 * @example
 * ```ts
 * async function zeroDesignSystemImport(): Promise<DesignSystemModule> {
 *   let module: DesignSystemModule;
 *   let type: ResourceType = ResourceType.remote;
 *   try {
 *     module = await import('foundationZero/ZeroDesignSystem');
 *     return assureDesignSystem(module);
 *   } catch (e) {
 *     logger.info(
 *       `Please note remoteEntry.js load errors are expected if module federated dependencies are offline. Falling back to locally bundled versions.`,
 *     );
 *     type = ResourceType.local;
 *     module = await import('@genesislcap/foundation-zero');
 *     return assureDesignSystem(module);
 *   } finally {
 *     logger.debug(`Using '${type}' version of foundation-zero`);
 *   }
 * }
 * ```
 *
 * @example You should also instruct webpack to use strict module handling.
 * ```ts
 * output: {
 *   strictModuleErrorHandling: true,
 *   strictModuleExceptionHandling: true,
 * }
 * ```
 *
 * @public
 */
export declare function assureDesignSystem(module: DesignSystemModule): DesignSystemModule;

/**
 * @internal
 */
export declare const BIG_INT_64_BITS = 64;

/**
 * @public
 */
export declare type ConstructableLifecycleHandler = Constructable<FASTElement & HTMLElement>;

/**
 * A mixin that provides functionality for raising `pending-state` events.
 * @public
 */
export declare type ConstructablePendingState = Constructable<FASTElement & HTMLElement>;

declare type Container = FoundationLayoutContainer | LayoutCacheContainer | DOMContainer;

/**
 * Represents the possible types of value conversion for data synchronization.
 * @public
 */
export declare type ConversionType = 'string' | 'number' | 'time' | 'boolean';

/**
 * A factory to create the error map.
 * @param logger - A logger error method reference.
 * @returns A ErrorMap instance.
 *
 * @public
 */
export declare const createErrorMap: <TErrorDetailMap extends ErrorDetailMap>(logger: ErrorMapLogger) => ErrorMap<TErrorDetailMap>;

/**
 * Creates a logger with the given name and options.
 * @param name - The name to give the logger.
 * @param options - The options to use when creating the logger.
 * @returns The resulting logger.
 *
 * @deprecated - Use `createLogger` from `@genesislcap/foundation-logger` instead.
 * @public
 */
export declare function createLogger(name: string, options?: LoggerOptions): Logger;

/**
 * Creates a new event observer instance.
 * @public
 */
export declare const createObserver: <EventType>() => Observer<EventType>;

/**
 * A Default Number Parser for deserializing objects.
 * @public
 */
export declare function customNumberParser(value: any): number | bigint;

/**
 * Represents a database with basic CRUD operations.
 * @public
 */
export declare interface Database<T extends DatabaseRecord> {
    isWorking: boolean;
    create(newValue: Omit<T, 'id'>): Promise<DatabaseAccessResult.Create<T>>;
    read(id: string): Promise<DatabaseAccessResult.Read<T>>;
    update(id: string, newValue: Omit<Partial<T>, 'id'>): Promise<DatabaseAccessResult.Update<T>>;
    delete(id: string): Promise<DatabaseAccessResult.Delete>;
    visit(visitor: (record: T) => void): Promise<void>;
    onBeforeUpdate(listener: Listener<DatabaseEvent.BeforeUpdate<T>>): () => void;
    onAfterUpdate(listener: Listener<DatabaseEvent.AfterUpdate<T>>): () => void;
}

/**
 * Namespace for database access result types.
 * @public
 */
export declare namespace DatabaseAccessResult {
    export interface Access<T> {
        value: T;
    }
    export interface Create<T> extends Access<T> {
    }
    export interface Read<T> extends Access<T> {
    }
    export interface Update<T> extends Access<T> {
    }
    export interface Delete {
        id: string;
        success: boolean;
    }
        {};
}

/**
 * Namespace for database events.
 * @public
 */
export declare namespace DatabaseEvent {
    export interface Update<T> {
        value: T;
    }
    export interface BeforeUpdate<T> extends Update<T> {
        newValue: T;
    }
    export interface AfterUpdate<T> extends Update<T> {
    }
        {};
}

/**
 * Represents a database record.
 * @public
 */
export declare interface DatabaseRecord {
    id: string;
}

/**
 * The default Organisation value, used in auth/login flow [`genesislcap-foundation-login`](https://link-to-docs).
 * @remarks Optional.
 * @public
 */
export declare let DEFAULT_ORGANISATION: string;

/**
 * The default Password value, used in auth/login flow [`genesislcap-foundation-login`](https://link-to-docs).
 * @remarks Optional.
 * @public
 */
export declare let DEFAULT_PASSWORD: string;

/**
 * The default Username value, used in auth/login flow [`genesislcap-foundation-login`](https://link-to-docs).
 * @remarks Optional.
 * @public
 */
export declare let DEFAULT_USER: string;

/**
 * @public
 */
export declare class DefaultErrorMap<TErrorDetailMap extends ErrorDetailMap> implements ErrorMap<TErrorDetailMap> {
    private logger;
    private map;
    /** {@inheritDoc ErrorMap.lastError} */
    lastError: Error;
    /**
     * @param logger - A logger error method reference.
     */
    constructor(logger: ErrorMapLogger);
    /** {@inheritDoc ErrorMap.set} */
    set(key: keyof TErrorDetailMap, error: Error): void;
    /**
     * Get an error by key.
     * @param key - The key.
     * @returns The error if it exists, otherwise undefined.
     *
     * @public
     */
    get(key: keyof TErrorDetailMap): Error | undefined;
    /**
     * Has an error for key.
     * @param key - The key.
     * @returns True if it exists, otherwise false.
     *
     * @public
     */
    has(key: keyof TErrorDetailMap): boolean;
    /**
     * Delete an error.
     * @param key - The key.
     * @returns True if successful, otherwise false.
     *
     * @public
     */
    delete(key: keyof TErrorDetailMap): boolean;
    /**
     * Clear errors.
     * @public
     */
    clear(): void;
    /**
     * The size of the error map.
     * @returns number
     *
     * @public
     */
    get size(): number;
    /**
     * @public
     */
    values(): IterableIterator<Error>;
    /** {@inheritDoc ErrorMap.messages} */
    get messages(): string;
}

/**
 * A map that associates specific HTML element tags with their corresponding default event names.
 * @public
 */
export declare const defaultEventMap: Map<string, EventName>;

/**
 * A Default JSONSerializer Config for serializing and deserializing functions.
 * @public
 */
export declare const defaultJSONSerializerConfig: JSONSerializerConfig;

/**
 * The default logger options.
 * @public
 */
export declare const defaultLoggerOptions: LoggerOptions;

/**
 * The default `ServerRowDTOMapper`.
 * @public
 */
export declare class DefaultServerRowDTOMapper implements ServerRowDTOMapper {
    /**
     * Converts a server row DTO to an entity.
     * @param dto - The DTO to convert.
     * @returns The resulting entity.
     * @public
     */
    fromDTO: (dto: ServerRowDTO) => ServerRowEntity;
    /**
     * Converts a server row entity to a DTO.
     * @param entity - The entity to convert.
     * @returns The resulting DTO.
     * @public
     */
    toDTO: (entity: ServerRowEntity) => ServerRowDTO;
}

/**
 * DesignSystemModule.
 * @public
 */
export declare interface DesignSystemModule {
    provideDesignSystem(element?: HTMLElement, prefix?: string): DesignSystem | Pick<DesignSystem, 'register'>;
    [key: string]: any;
}

/**
 * DesignSystemResource.
 * @public
 */
export declare type DesignSystemResource<T = DesignSystemModule> = Promise<T>;

/**
 * Signifies the element is placed on the DOM, not inside any dynamic layout components
 * @internal
 */
export declare type DOMContainer = {
    _key: 'dom';
};

/**
 * @public
 */
export declare interface DTOMapper<TDTO, TEntity> {
    fromArgs?: (...args: any[]) => TEntity;
    fromDTO: (dto: TDTO) => TEntity;
    toDTO: (entity: Partial<TEntity>) => TDTO;
}

/**
 * @public
 */
export declare type ErrorDetailMap = Record<string, unknown>;

/**
 * @public
 */
export declare interface ErrorMap<TErrorDetailMap extends ErrorDetailMap> extends Pick<Map<keyof TErrorDetailMap, Error>, 'get' | 'has' | 'delete' | 'clear' | 'size' | 'values'> {
    /**
     * Error map as messages.
     * @returns string
     *
     * @public
     */
    readonly messages: string;
    /**
     * Set an error by key.
     * @param key - The key.
     * @param error - The error.
     *
     * @public
     */
    set(key: keyof TErrorDetailMap, error: Error): void;
    /**
     * The last set error if any.
     * @returns Error
     *
     * @public
     */
    readonly lastError: Error;
}

/**
 * @public
 */
export declare type ErrorMapLogger = (...args: any[]) => void;

/**
 * Represents the possible event names for data synchronization.
 * @public
 */
export declare type EventName = 'input' | 'change' | 'default';

/**
 * An enum of possible font styles.
 * @public
 */
export declare enum FontStyle {
    Italic = "italic",
    Normal = "normal"
}

/**
 * An enum of possible font weights.
 * @public
 */
export declare enum FontWeight {
    Thin = 100,
    Light = 300,
    Regular = 400,
    Medium = 500,
    Bold = 700,
    Black = 900
}

/**
 * The path to a JSON config file for the HTTP mode.
 * @remarks Optional.
 * @public
 */
export declare let FORCE_HTTP: string;

/**
 * Formats [DATE] UNIX Timestamps (without time) to readable strings
 * @public
 */
export declare function formatDateTimestamp(timestamp: number): string;

/**
 * Formats [DATETIME] UNIX Timestamps (with time) to readable strings
 * @public
 */
export declare function formatDateTimeTimestamp(timestamp: number): string;

/**
 * Formats [DATE|DATETIME] Unix Timestamps to readable strings
 * @public
 * @param timestamp - The UNIX Timestamp.
 * @param withTime - The flag to determine if formatted value should contain 'time' info.
 */
export declare function formatTimestamp(timestamp: number, withTime: boolean): string;

/**
 * Minimum interface to interact with FoundationLayout
 * without importing it as a dependency of `@genesislcap/foundation-utils`
 * @internal
 */
export declare type FoundationLayoutContainer = {
    dragging: boolean;
    hasFirstLoaded: boolean;
    lifecycleUpdateToken: string | undefined;
    _key: 'foundation-layout';
};

/**
 * Genesis Socket URL
 * @remarks
 * Built from the current {@link https://developer.mozilla.org/en-US/docs/Web/API/Location} object and {@link SOCKET_EXT | socket extension}.
 * @example
 * ```ts
 * await this.connect.connect(GENESIS_SOCKET_URL); // < wss://localhost:6060/gwf/
 * ```
 * @public
 */
export declare const GENESIS_SOCKET_URL: string;

/**
 * @public
 */
export declare function getDateFormatter(locale?: string, options?: Intl.DateTimeFormatOptions): (params: any) => string;

/**
 * Generates a CSS mixin for the specified font family, style, and weight.
 * @param family - The font family.
 * @param style - Optional. The font style. Defaults to FontStyle.Normal.
 * @param weight - Optional. The font weight. Defaults to FontWeight.Regular.
 * @returns The generated CSS mixin.
 * @public
 */
export declare const getFontMixin: (family: string, style?: FontStyle, weight?: FontWeight) => string;

/**
 * @public
 */
export declare function getNumberFormatter(format: string, locale?: string | null): (params: any) => any;

/**
 * Configuration settings for HTTP, used in http connect flow [`genesislcap-foundation-comms.HttpConnectConfig`](https://link-to-docs).
 * @remarks Optional.
 * @public
 */
export declare let HTTP_CONFIG: string;

/**
 * Returns a boolean value indicating whether the current window is inside an iframe.
 * @returns true if the window is inside an iframe, false otherwise.
 * @public
 */
export declare const inIFrame: () => boolean;

/**
 * An in memory database of specific DatabaseRecord types.
 * @public
 */
export declare class InMemoryDatabase<T extends DatabaseRecord> implements Database<T> {
    private uuid;
    isWorking: boolean;
    private records;
    private beforeUpdateListeners;
    private afterUpdateListeners;
    constructor(uuid: UUID);
    create(newValue: Omit<T, 'id'>): Promise<DatabaseAccessResult.Create<T>>;
    read(id: string): Promise<DatabaseAccessResult.Read<T>>;
    update(id: string, newValue: Omit<Partial<T>, 'id'>): Promise<DatabaseAccessResult.Update<T>>;
    delete(id: string): Promise<DatabaseAccessResult.Delete>;
    visit(visitor: (record: T) => void): Promise<void>;
    onBeforeUpdate(listener: Listener<DatabaseEvent.BeforeUpdate<T>>): () => void;
    onAfterUpdate(listener: Listener<DatabaseEvent.AfterUpdate<T>>): () => void;
}

/**
 * Inserts a CSS rule into the document by creating a new style element or using an existing one with the specified ID.
 * Returns a function that can be called to remove the rule from the document.
 * @public
 * @param cssRule - The CSS rule to insert.
 * @param styleElementId - The ID of the style element to use or create.
 * @returns A function that removes the rule from the document.
 */
export declare const insertDocumentCSSRule: (cssRule: string, styleElementId: string) => (() => void);

/**
 * Inserts a CSS link into the document if it doesn't already exist.
 * @public
 * @param href - The URL of the CSS file to insert.
 */
export declare const insertDocumentLink: (href: string) => void;

/**
 * Returns a boolean value indicating whether the user is running the Symphony desktop app.
 * @returns true if the user is running the Symphony desktop app, false otherwise.
 * @public
 */
export declare const inSymphonyDesktop: () => boolean;

/**
 * Determines if the current environment is a development environment.
 * @public
 */
export declare const isDev: () => boolean;

/**
 * JSON replacer function.
 * @param key - The object key.
 * @param value - The key value.
 * @returns The resulting value.
 * @public
 */
export declare function JSONReplacer(key: string, value: any): any;

/**
 * JSON reviver function.
 * @param key - The object key.
 * @param value - The key value.
 * @returns The resulting value.
 * @public
 */
export declare function JSONReviver(key: string, value: any): any;

/**
 * An interface representing a JSON serializer.
 * @public
 */
export declare interface JSONSerializer {
    /**
     * Serializes an object to a JSON string.
     * @param object - The object to serialize.
     * @returns The serialized JSON string.
     */
    serialize(object: any): string;
    /**
     * Deserializes a response object from a HTTP request to a JavaScript object.
     * @param response - The HTTP response object.
     * @returns A promise that resolves to the deserialized JavaScript object.
     */
    deserialize<T = any>(response: Response): Promise<T>;
    /**
     * Deserializes a message event object to a JavaScript object.
     * @param event - The message event object.
     * @returns The deserialized JavaScript object.
     */
    deserialize<T = any>(event: MessageEvent): T;
    /**
     * Deserializes a JSON string to a JavaScript object.
     * @param text - The JSON string to deserialize.
     * @returns The deserialized JavaScript object.
     */
    deserialize<T = any>(text: string): T;
}

/**
 * A DI token for the JSON serializer.
 * @public
 */
export declare const JSONSerializer: InterfaceSymbol<JSONSerializer>;

/**
 * Configuration options for the JSONSerializer Config instance.
 * @public
 */
export declare interface JSONSerializerConfig {
    parse(text: string, reviver?: (this: any, key: string, value: any) => any): any;
    parse(text: string, reviver?: (this: any, key: string, value: any) => any, parseNumber?: any): any;
    stringify(value: any, replacer?: (this: any, key: string, value: any) => any, space?: string | number): string;
    stringify(value: any, replacer?: (number | string)[] | null, space?: string | number): string;
}

/**
 * Configuration options for the JSONSerializer Config instance.
 * @internal
 */
export declare const JSONSerializerConfig: InterfaceSymbol<JSONSerializerConfig>;

/**
 * Signifies the internal document fragment that stores the master copy
 * of the layout item, not part of the DOM.
 * @internal
 */
export declare type LayoutCacheContainer = {
    _key: 'foundation-layout-cache';
};

/**
 * Stored on the layout's internal cache to signify that the document is not part of the DOM
 * @public
 */
export declare const layoutCacheDocument: unique symbol;

/**
 * @beta
 * Mixin class to expose `shouldRunConnect` and `shouldRunDisconnect`
 *
 * @remarks These can be used to control what lifecycle functionality is ran when the elements
 * are part of a custom layout. The class itself handles all events
 * @privateRemarks
 * #_ syntax is used for a javascript private method/variable
 * we can't use private/protected from typescript in an anonymous class like this
 */
export declare const LifecycleMixin: <T extends ConstructableLifecycleHandler>(Base: T) => {
    new (...args: any[]): {
        /** @internal **/
        "__#1@#_container": Container;
        /**
         * If the token changes we suggest blocking the lifecycle methods
         * @internal
         **/
        "__#1@#_latestTokenCode": string;
        /**
         * Track first loaded to allow us to always run the connect lifecycle on the first load
         * @internal
         **/
        "__#1@#_hasFirstLoaded": boolean;
        /**
         * When you request to delete an item from the layout it will be removed from the DOM,
         * but the lifecycle might be blocked due to the same mechanism that we block the other
         * items.
         * This would block cleanup of the item, so we need to force a disconnection after a timeout
         * if this item hasn't reconnected within `CLEANUP_TIMEOUT_MS`.
         * @internal
         **/
        "__#1@#_cleanupTimeout": NodeJS.Timeout;
        /**
         * Used to force lifecycle to run even if the token has changed, such as in instances where
         * we are forcing a disconnection due to this item being remove from the layout.
         * @internal
         **/
        "__#1@#_shouldForceLifecycle": boolean;
        /**
         * @privateRemarks
         * Rather than using the basic implementation of cloning we run a copy constructor (deepClone)
         * and then manually clone the children too so they can intercept with an overriden cloneNode
         * if they want to
         */
        cloneNode(deep?: boolean): Node;
        /**
         * @privateRemarks
         * Basic implementation of a copy constructor which creates a new element of the same class
         * and copies all attributes over.
         * None-trivial elements will likely want to override this function, call it using super.deepClone(),
         * and then perform extra setup such as copying properties, setting up event listeners
         */
        deepClone(): Node;
        /**
         * @beta
         * @returns - boolean controlling whether to run all disconnectedCallback lifecycle functionality
         */
        readonly shouldRunDisconnect: boolean;
        /**
         * @beta
         * @returns - boolean controlling whether to run all connectedCallback lifecycle functionality
         * @privateRemarks
         * For now its just the same logic as shouldRunDisconnect
         */
        readonly shouldRunConnect: boolean;
        "__#1@#_blockLifecycleDueToTokenChange"(lifecycleType: Lifecycletype): boolean;
        /**
         * @internal
         * Recursive function to try and find containing layout object
         *
         * @returns the layout object casted to the {@link @genesislcap/foundation-utils#FoundationLayoutContainer} interface if we are in a
         * layout object, {@link @genesislcap/foundation-utils#LayoutCacheContainer} if the element is in the layout cache, else {@link @genesislcap/foundation-utils#DOMContainer}.
         */
        "__#1@#_tryFindContainingLayout"(e: Element): Container;
        connectedCallback(): void;
        readonly $fastController: Controller;
        $emit(type: string, detail?: any, options?: Omit<CustomEventInit<any>, "detail">): boolean | void;
        disconnectedCallback(): void;
        attributeChangedCallback(name: string, oldValue: string, newValue: string): void;
        accessKey: string;
        readonly accessKeyLabel: string;
        autocapitalize: string;
        dir: string;
        draggable: boolean;
        hidden: boolean;
        inert: boolean;
        innerText: string;
        lang: string;
        readonly offsetHeight: number;
        readonly offsetLeft: number;
        readonly offsetParent: Element;
        readonly offsetTop: number;
        readonly offsetWidth: number;
        outerText: string;
        spellcheck: boolean;
        title: string;
        translate: boolean;
        attachInternals(): ElementInternals;
        click(): void;
        addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;
        addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;
        removeEventListener<K_1 extends keyof HTMLElementEventMap>(type: K_1, listener: (this: HTMLElement, ev: HTMLElementEventMap[K_1]) => any, options?: boolean | EventListenerOptions): void;
        removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;
        readonly attributes: NamedNodeMap;
        readonly classList: DOMTokenList;
        className: string;
        readonly clientHeight: number;
        readonly clientLeft: number;
        readonly clientTop: number;
        readonly clientWidth: number;
        id: string;
        readonly localName: string;
        readonly namespaceURI: string;
        onfullscreenchange: (this: Element, ev: Event) => any;
        onfullscreenerror: (this: Element, ev: Event) => any;
        outerHTML: string;
        readonly ownerDocument: Document;
        readonly part: DOMTokenList;
        readonly prefix: string;
        readonly scrollHeight: number;
        scrollLeft: number;
        scrollTop: number;
        readonly scrollWidth: number;
        readonly shadowRoot: ShadowRoot;
        slot: string;
        readonly tagName: string;
        attachShadow(init: ShadowRootInit): ShadowRoot;
        closest<K_2 extends keyof HTMLElementTagNameMap>(selector: K_2): HTMLElementTagNameMap[K_2];
        closest<K_3 extends keyof SVGElementTagNameMap>(selector: K_3): SVGElementTagNameMap[K_3];
        closest<E extends Element = Element>(selectors: string): E;
        getAttribute(qualifiedName: string): string;
        getAttributeNS(namespace: string, localName: string): string;
        getAttributeNames(): string[];
        getAttributeNode(qualifiedName: string): Attr;
        getAttributeNodeNS(namespace: string, localName: string): Attr;
        getBoundingClientRect(): DOMRect;
        getClientRects(): DOMRectList;
        getElementsByClassName(classNames: string): HTMLCollectionOf<Element>;
        getElementsByTagName<K_4 extends keyof HTMLElementTagNameMap>(qualifiedName: K_4): HTMLCollectionOf<HTMLElementTagNameMap[K_4]>;
        getElementsByTagName<K_5 extends keyof SVGElementTagNameMap>(qualifiedName: K_5): HTMLCollectionOf<SVGElementTagNameMap[K_5]>;
        getElementsByTagName(qualifiedName: string): HTMLCollectionOf<Element>;
        getElementsByTagNameNS(namespaceURI: "http://www.w3.org/1999/xhtml", localName: string): HTMLCollectionOf<HTMLElement>;
        getElementsByTagNameNS(namespaceURI: "http://www.w3.org/2000/svg", localName: string): HTMLCollectionOf<SVGElement>;
        getElementsByTagNameNS(namespace: string, localName: string): HTMLCollectionOf<Element>;
        hasAttribute(qualifiedName: string): boolean;
        hasAttributeNS(namespace: string, localName: string): boolean;
        hasAttributes(): boolean;
        hasPointerCapture(pointerId: number): boolean;
        insertAdjacentElement(where: InsertPosition, element: Element): Element;
        insertAdjacentHTML(position: InsertPosition, text: string): void;
        insertAdjacentText(where: InsertPosition, data: string): void;
        matches(selectors: string): boolean;
        releasePointerCapture(pointerId: number): void;
        removeAttribute(qualifiedName: string): void;
        removeAttributeNS(namespace: string, localName: string): void;
        removeAttributeNode(attr: Attr): Attr;
        requestFullscreen(options?: FullscreenOptions): Promise<void>;
        requestPointerLock(): void;
        scroll(options?: ScrollToOptions): void;
        scroll(x: number, y: number): void;
        scrollBy(options?: ScrollToOptions): void;
        scrollBy(x: number, y: number): void;
        scrollIntoView(arg?: boolean | ScrollIntoViewOptions): void;
        scrollTo(options?: ScrollToOptions): void;
        scrollTo(x: number, y: number): void;
        setAttribute(qualifiedName: string, value: string): void;
        setAttributeNS(namespace: string, qualifiedName: string, value: string): void;
        setAttributeNode(attr: Attr): Attr;
        setAttributeNodeNS(attr: Attr): Attr;
        setPointerCapture(pointerId: number): void;
        toggleAttribute(qualifiedName: string, force?: boolean): boolean;
        webkitMatchesSelector(selectors: string): boolean;
        readonly baseURI: string;
        readonly childNodes: NodeListOf<ChildNode>;
        readonly firstChild: ChildNode;
        readonly isConnected: boolean;
        readonly lastChild: ChildNode;
        readonly nextSibling: ChildNode;
        readonly nodeName: string;
        readonly nodeType: number;
        nodeValue: string;
        readonly parentElement: HTMLElement;
        readonly parentNode: ParentNode;
        readonly previousSibling: ChildNode;
        textContent: string;
        appendChild<T_1 extends Node>(node: T_1): T_1;
        compareDocumentPosition(other: Node): number;
        contains(other: Node): boolean;
        getRootNode(options?: GetRootNodeOptions): Node;
        hasChildNodes(): boolean;
        insertBefore<T_2 extends Node>(node: T_2, child: Node): T_2;
        isDefaultNamespace(namespace: string): boolean;
        isEqualNode(otherNode: Node): boolean;
        isSameNode(otherNode: Node): boolean;
        lookupNamespaceURI(prefix: string): string;
        lookupPrefix(namespace: string): string;
        normalize(): void;
        removeChild<T_3 extends Node>(child: T_3): T_3;
        replaceChild<T_4 extends Node>(node: Node, child: T_4): T_4;
        readonly ATTRIBUTE_NODE: number;
        readonly CDATA_SECTION_NODE: number;
        readonly COMMENT_NODE: number;
        readonly DOCUMENT_FRAGMENT_NODE: number;
        readonly DOCUMENT_NODE: number;
        readonly DOCUMENT_POSITION_CONTAINED_BY: number;
        readonly DOCUMENT_POSITION_CONTAINS: number;
        readonly DOCUMENT_POSITION_DISCONNECTED: number;
        readonly DOCUMENT_POSITION_FOLLOWING: number;
        readonly DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC: number;
        readonly DOCUMENT_POSITION_PRECEDING: number;
        readonly DOCUMENT_TYPE_NODE: number;
        readonly ELEMENT_NODE: number;
        readonly ENTITY_NODE: number;
        readonly ENTITY_REFERENCE_NODE: number;
        readonly NOTATION_NODE: number;
        readonly PROCESSING_INSTRUCTION_NODE: number;
        readonly TEXT_NODE: number;
        dispatchEvent(event: Event): boolean;
        ariaAtomic: string;
        ariaAutoComplete: string;
        ariaBusy: string;
        ariaChecked: string;
        ariaColCount: string;
        ariaColIndex: string;
        ariaColIndexText: string;
        ariaColSpan: string;
        ariaCurrent: string;
        ariaDisabled: string;
        ariaExpanded: string;
        ariaHasPopup: string;
        ariaHidden: string;
        ariaInvalid: string;
        ariaKeyShortcuts: string;
        ariaLabel: string;
        ariaLevel: string;
        ariaLive: string;
        ariaModal: string;
        ariaMultiLine: string;
        ariaMultiSelectable: string;
        ariaOrientation: string;
        ariaPlaceholder: string;
        ariaPosInSet: string;
        ariaPressed: string;
        ariaReadOnly: string;
        ariaRequired: string;
        ariaRoleDescription: string;
        ariaRowCount: string;
        ariaRowIndex: string;
        ariaRowIndexText: string;
        ariaRowSpan: string;
        ariaSelected: string;
        ariaSetSize: string;
        ariaSort: string;
        ariaValueMax: string;
        ariaValueMin: string;
        ariaValueNow: string;
        ariaValueText: string;
        role: string;
        animate(keyframes: Keyframe[] | PropertyIndexedKeyframes, options?: number | KeyframeAnimationOptions): Animation;
        getAnimations(options?: GetAnimationsOptions): Animation[];
        after(...nodes: (string | Node)[]): void;
        before(...nodes: (string | Node)[]): void;
        remove(): void;
        replaceWith(...nodes: (string | Node)[]): void;
        innerHTML: string;
        readonly nextElementSibling: Element;
        readonly previousElementSibling: Element;
        readonly childElementCount: number;
        readonly children: HTMLCollection;
        readonly firstElementChild: Element;
        readonly lastElementChild: Element;
        append(...nodes: (string | Node)[]): void;
        prepend(...nodes: (string | Node)[]): void;
        querySelector<K_6 extends keyof HTMLElementTagNameMap>(selectors: K_6): HTMLElementTagNameMap[K_6];
        querySelector<K_7 extends keyof SVGElementTagNameMap>(selectors: K_7): SVGElementTagNameMap[K_7];
        querySelector<E_1 extends Element = Element>(selectors: string): E_1;
        querySelectorAll<K_8 extends keyof HTMLElementTagNameMap>(selectors: K_8): NodeListOf<HTMLElementTagNameMap[K_8]>;
        querySelectorAll<K_9 extends keyof SVGElementTagNameMap>(selectors: K_9): NodeListOf<SVGElementTagNameMap[K_9]>;
        querySelectorAll<E_2 extends Element = Element>(selectors: string): NodeListOf<E_2>;
        replaceChildren(...nodes: (string | Node)[]): void;
        readonly assignedSlot: HTMLSlotElement;
        oncopy: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
        oncut: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
        onpaste: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
        readonly style: CSSStyleDeclaration;
        contentEditable: string;
        enterKeyHint: string;
        inputMode: string;
        readonly isContentEditable: boolean;
        onabort: (this: GlobalEventHandlers, ev: UIEvent) => any;
        onanimationcancel: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
        onanimationend: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
        onanimationiteration: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
        onanimationstart: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
        onauxclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onbeforeinput: (this: GlobalEventHandlers, ev: InputEvent) => any;
        onblur: (this: GlobalEventHandlers, ev: FocusEvent) => any;
        oncancel: (this: GlobalEventHandlers, ev: Event) => any;
        oncanplay: (this: GlobalEventHandlers, ev: Event) => any;
        oncanplaythrough: (this: GlobalEventHandlers, ev: Event) => any;
        onchange: (this: GlobalEventHandlers, ev: Event) => any;
        onclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onclose: (this: GlobalEventHandlers, ev: Event) => any;
        oncontextmenu: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        oncuechange: (this: GlobalEventHandlers, ev: Event) => any;
        ondblclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        ondrag: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondragend: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondragenter: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondragleave: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondragover: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondragstart: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondrop: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondurationchange: (this: GlobalEventHandlers, ev: Event) => any;
        onemptied: (this: GlobalEventHandlers, ev: Event) => any;
        onended: (this: GlobalEventHandlers, ev: Event) => any;
        onerror: OnErrorEventHandlerNonNull;
        onfocus: (this: GlobalEventHandlers, ev: FocusEvent) => any;
        onformdata: (this: GlobalEventHandlers, ev: FormDataEvent) => any;
        ongotpointercapture: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        oninput: (this: GlobalEventHandlers, ev: Event) => any;
        oninvalid: (this: GlobalEventHandlers, ev: Event) => any;
        onkeydown: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
        onkeypress: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
        onkeyup: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
        onload: (this: GlobalEventHandlers, ev: Event) => any;
        onloadeddata: (this: GlobalEventHandlers, ev: Event) => any;
        onloadedmetadata: (this: GlobalEventHandlers, ev: Event) => any;
        onloadstart: (this: GlobalEventHandlers, ev: Event) => any;
        onlostpointercapture: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onmousedown: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmouseenter: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmouseleave: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmousemove: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmouseout: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmouseover: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmouseup: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onpause: (this: GlobalEventHandlers, ev: Event) => any;
        onplay: (this: GlobalEventHandlers, ev: Event) => any;
        onplaying: (this: GlobalEventHandlers, ev: Event) => any;
        onpointercancel: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerdown: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerenter: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerleave: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointermove: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerout: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerover: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerup: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onprogress: (this: GlobalEventHandlers, ev: ProgressEvent<EventTarget>) => any;
        onratechange: (this: GlobalEventHandlers, ev: Event) => any;
        onreset: (this: GlobalEventHandlers, ev: Event) => any;
        onresize: (this: GlobalEventHandlers, ev: UIEvent) => any;
        onscroll: (this: GlobalEventHandlers, ev: Event) => any;
        onsecuritypolicyviolation: (this: GlobalEventHandlers, ev: SecurityPolicyViolationEvent) => any;
        onseeked: (this: GlobalEventHandlers, ev: Event) => any;
        onseeking: (this: GlobalEventHandlers, ev: Event) => any;
        onselect: (this: GlobalEventHandlers, ev: Event) => any;
        onselectionchange: (this: GlobalEventHandlers, ev: Event) => any;
        onselectstart: (this: GlobalEventHandlers, ev: Event) => any;
        onslotchange: (this: GlobalEventHandlers, ev: Event) => any;
        onstalled: (this: GlobalEventHandlers, ev: Event) => any;
        onsubmit: (this: GlobalEventHandlers, ev: SubmitEvent) => any;
        onsuspend: (this: GlobalEventHandlers, ev: Event) => any;
        ontimeupdate: (this: GlobalEventHandlers, ev: Event) => any;
        ontoggle: (this: GlobalEventHandlers, ev: Event) => any;
        ontouchcancel?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
        ontouchend?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
        ontouchmove?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
        ontouchstart?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
        ontransitioncancel: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
        ontransitionend: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
        ontransitionrun: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
        ontransitionstart: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
        onvolumechange: (this: GlobalEventHandlers, ev: Event) => any;
        onwaiting: (this: GlobalEventHandlers, ev: Event) => any;
        onwebkitanimationend: (this: GlobalEventHandlers, ev: Event) => any;
        onwebkitanimationiteration: (this: GlobalEventHandlers, ev: Event) => any;
        onwebkitanimationstart: (this: GlobalEventHandlers, ev: Event) => any;
        onwebkittransitionend: (this: GlobalEventHandlers, ev: Event) => any;
        onwheel: (this: GlobalEventHandlers, ev: WheelEvent) => any;
        autofocus: boolean;
        readonly dataset: DOMStringMap;
        nonce?: string;
        tabIndex: number;
        blur(): void;
        focus(options?: FocusOptions): void;
    };
} & T;

declare type Lifecycletype = 'connect' | 'disconnect' | 'reconnect';

/**
 * Creates an observer that facilitates the subscription and publication of events.
 * @public
 */
export declare type Listener<EventType> = (ev: EventType) => void;

/**
 * Loads font faces by inserting a style element with the specified font face rules into the document.
 * @param fontFaceRules - The CSS rules for the font faces.
 * @param styleElementId - The ID of the style element to insert into the document.
 * @public
 */
export declare const loadFontFaces: (fontFaceRules: string, styleElementId: string) => void;

/**
 * A logger that extends the `Consola` logger.
 *
 * @remarks
 * This logger is part of the package's API but may still be subject to changes and improvements.
 *
 * @public
 */
export declare interface Logger extends Logger_2 {
    /**
     * A logger method for deprecated symbols.
     *
     * @remarks
     * Pair with a deprecated tag. See {@link https://tsdoc.org/pages/tags/deprecated/} for more information.
     *
     * @example
     * ```ts
     * logger.deprecated('someSymbol');
     * ```
     *
     * @example With a custom instruction and target removal version.
     * ```ts
     * logger.deprecated('routeButtons', 'Use the default slot instead.', '20.0.0');
     * ```
     *
     * @param symbol - The deprecated symbol.
     * @param instruction - An optional alternative instruction. Defaults to `Consult docs for better alternative.`
     * @param removalVersionTarget - An optional target removal version number.
     */
    deprecated(symbol: string, instruction?: string, removalVersionTarget?: string): void;
}

/**
 * Logger for the foundation-utils package
 * @public
 */
export declare const logger: Logger_2;

/**
 * Options for creating a logger.
 * @public
 */
export declare interface LoggerOptions extends LoggerOptions_2 {
}

export { LogLevel }

/**
 * @internal
 */
declare type LowercaseOperation = 'insert' | 'modify' | 'delete';

/**
 * @public
 */
export declare class NumberParser {
    /**
     * reference: https://observablehq.com/\@mbostock/localized-number-parsing
     * @internal
     */
    private _group;
    private _decimal;
    private _separator;
    private _numeral;
    private _index;
    constructor(locale: string);
    parse(localeNumber: string): number;
    hasSeparator(localeNumber: string): boolean;
}

/**
 * @privateRemarks
 * Code borrowed from fast-element v2. We can remove post upgrade.
 *
 * @example
 * Reactive visitor example.
 * ```ts
 * const makeObserverVisitor: ObjectVisitor<undefined> = {
 *   visitObject: noop,
 *   visitArray: noop,
 *   visitProperty(object: any, propertyName: string, value: any): void {
 *     Reflect.defineProperty(object, propertyName, {
 *       enumerable: true,
 *       get() {
 *         Observable.track(object, propertyName);
 *         return value;
 *       },
 *       set(newValue: any) {
 *         if (value !== newValue) {
 *           value = newValue;
 *           Observable.notify(object, propertyName);
 *         }
 *       },
 *     });
 *   },
 * };
 * ```
 *
 * @beta
 */
export declare interface ObjectVisitor<TVisitorData> {
    visitObject(object: any, data: TVisitorData): void;
    visitArray(array: any[], data: TVisitorData): void;
    visitProperty(object: any, key: PropertyKey, value: any, data: TVisitorData): void;
}

/**
 * Represents an event observer that manages the subscription and publication of events.
 * @public
 */
export declare interface Observer<EventType> {
    subscribe: Subscribe<EventType>;
    publish: Publish<EventType>;
}

/**
 * Opens a new browser window with the specified URL, target, width, and height.
 * @param urlNavigate - The URL to navigate to.
 * @param target - The name of the new window.
 * @param popUpWidth - The width of the new window (optional).
 * @param popUpHeight - The height of the new window (optional).
 * @returns A reference to the new window.
 * @public
 */
export declare const openPopup: (urlNavigate: string, target: string, popUpWidth?: number, popUpHeight?: number) => Window;

/**
 * The `PendingState` mixin.
 * @public
 */
export declare const PendingState: <TBase extends ConstructablePendingState>(Base: TBase) => {
    new (...args: any[]): {
        /**
         * The number of promises that are currently pending.
         * @public
         */
        pendingCount: number;
        /**
         * The number of promises that have been resolved.
         * @public
         */
        resolvedCount: number;
        /**
         * A boolean indicating whether there are any pending children.
         * @public
         */
        hasPendingChildren: boolean;
        /**
         * Gets the progress of the pending promises as a percentage between 0 and 1.
         * @public
         */
        readonly progress: number;
        /**
         * Called when the element is connected to the DOM.
         * @internal
         */
        connectedCallback(): void;
        /**
         * Called when the element is disconnected from the DOM.
         * @internal
         */
        disconnectedCallback(): void;
        /**
         * Handles the `pending-state` event.
         * @param event - The `pending-state` event.
         * @internal
         */
        onPendingState({ detail }: PendingStateEvent): Promise<void>;
        readonly $fastController: Controller;
        $emit(type: string, detail?: any, options?: Omit<CustomEventInit<any>, "detail">): boolean | void;
        attributeChangedCallback(name: string, oldValue: string, newValue: string): void;
        accessKey: string;
        readonly accessKeyLabel: string;
        autocapitalize: string;
        dir: string;
        draggable: boolean;
        hidden: boolean;
        inert: boolean;
        innerText: string;
        lang: string;
        readonly offsetHeight: number;
        readonly offsetLeft: number;
        readonly offsetParent: Element;
        readonly offsetTop: number;
        readonly offsetWidth: number;
        outerText: string;
        spellcheck: boolean;
        title: string;
        translate: boolean;
        attachInternals(): ElementInternals;
        click(): void;
        addEventListener<K extends keyof HTMLElementEventMap>(type: K, listener: (this: HTMLElement, ev: HTMLElementEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;
        addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;
        removeEventListener<K_1 extends keyof HTMLElementEventMap>(type: K_1, listener: (this: HTMLElement, ev: HTMLElementEventMap[K_1]) => any, options?: boolean | EventListenerOptions): void;
        removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;
        readonly attributes: NamedNodeMap;
        readonly classList: DOMTokenList;
        className: string;
        readonly clientHeight: number;
        readonly clientLeft: number;
        readonly clientTop: number;
        readonly clientWidth: number;
        id: string;
        readonly localName: string;
        readonly namespaceURI: string;
        onfullscreenchange: (this: Element, ev: Event) => any;
        onfullscreenerror: (this: Element, ev: Event) => any;
        outerHTML: string;
        readonly ownerDocument: Document;
        readonly part: DOMTokenList;
        readonly prefix: string;
        readonly scrollHeight: number;
        scrollLeft: number;
        scrollTop: number;
        readonly scrollWidth: number;
        readonly shadowRoot: ShadowRoot;
        slot: string;
        readonly tagName: string;
        attachShadow(init: ShadowRootInit): ShadowRoot;
        closest<K_2 extends keyof HTMLElementTagNameMap>(selector: K_2): HTMLElementTagNameMap[K_2];
        closest<K_3 extends keyof SVGElementTagNameMap>(selector: K_3): SVGElementTagNameMap[K_3];
        closest<E extends Element = Element>(selectors: string): E;
        getAttribute(qualifiedName: string): string;
        getAttributeNS(namespace: string, localName: string): string;
        getAttributeNames(): string[];
        getAttributeNode(qualifiedName: string): Attr;
        getAttributeNodeNS(namespace: string, localName: string): Attr;
        getBoundingClientRect(): DOMRect;
        getClientRects(): DOMRectList;
        getElementsByClassName(classNames: string): HTMLCollectionOf<Element>;
        getElementsByTagName<K_4 extends keyof HTMLElementTagNameMap>(qualifiedName: K_4): HTMLCollectionOf<HTMLElementTagNameMap[K_4]>;
        getElementsByTagName<K_5 extends keyof SVGElementTagNameMap>(qualifiedName: K_5): HTMLCollectionOf<SVGElementTagNameMap[K_5]>;
        getElementsByTagName(qualifiedName: string): HTMLCollectionOf<Element>;
        getElementsByTagNameNS(namespaceURI: "http://www.w3.org/1999/xhtml", localName: string): HTMLCollectionOf<HTMLElement>;
        getElementsByTagNameNS(namespaceURI: "http://www.w3.org/2000/svg", localName: string): HTMLCollectionOf<SVGElement>;
        getElementsByTagNameNS(namespace: string, localName: string): HTMLCollectionOf<Element>;
        hasAttribute(qualifiedName: string): boolean;
        hasAttributeNS(namespace: string, localName: string): boolean;
        hasAttributes(): boolean;
        hasPointerCapture(pointerId: number): boolean;
        insertAdjacentElement(where: InsertPosition, element: Element): Element;
        insertAdjacentHTML(position: InsertPosition, text: string): void;
        insertAdjacentText(where: InsertPosition, data: string): void;
        matches(selectors: string): boolean;
        releasePointerCapture(pointerId: number): void;
        removeAttribute(qualifiedName: string): void;
        removeAttributeNS(namespace: string, localName: string): void;
        removeAttributeNode(attr: Attr): Attr;
        requestFullscreen(options?: FullscreenOptions): Promise<void>;
        requestPointerLock(): void;
        scroll(options?: ScrollToOptions): void;
        scroll(x: number, y: number): void;
        scrollBy(options?: ScrollToOptions): void;
        scrollBy(x: number, y: number): void;
        scrollIntoView(arg?: boolean | ScrollIntoViewOptions): void;
        scrollTo(options?: ScrollToOptions): void;
        scrollTo(x: number, y: number): void;
        setAttribute(qualifiedName: string, value: string): void;
        setAttributeNS(namespace: string, qualifiedName: string, value: string): void;
        setAttributeNode(attr: Attr): Attr;
        setAttributeNodeNS(attr: Attr): Attr;
        setPointerCapture(pointerId: number): void;
        toggleAttribute(qualifiedName: string, force?: boolean): boolean;
        webkitMatchesSelector(selectors: string): boolean;
        readonly baseURI: string;
        readonly childNodes: NodeListOf<ChildNode>;
        readonly firstChild: ChildNode;
        readonly isConnected: boolean;
        readonly lastChild: ChildNode;
        readonly nextSibling: ChildNode;
        readonly nodeName: string;
        readonly nodeType: number;
        nodeValue: string;
        readonly parentElement: HTMLElement;
        readonly parentNode: ParentNode;
        readonly previousSibling: ChildNode;
        textContent: string;
        appendChild<T extends Node>(node: T): T;
        cloneNode(deep?: boolean): Node;
        compareDocumentPosition(other: Node): number;
        contains(other: Node): boolean;
        getRootNode(options?: GetRootNodeOptions): Node;
        hasChildNodes(): boolean;
        insertBefore<T_1 extends Node>(node: T_1, child: Node): T_1;
        isDefaultNamespace(namespace: string): boolean;
        isEqualNode(otherNode: Node): boolean;
        isSameNode(otherNode: Node): boolean;
        lookupNamespaceURI(prefix: string): string;
        lookupPrefix(namespace: string): string;
        normalize(): void;
        removeChild<T_2 extends Node>(child: T_2): T_2;
        replaceChild<T_3 extends Node>(node: Node, child: T_3): T_3;
        readonly ATTRIBUTE_NODE: number;
        readonly CDATA_SECTION_NODE: number;
        readonly COMMENT_NODE: number;
        readonly DOCUMENT_FRAGMENT_NODE: number;
        readonly DOCUMENT_NODE: number;
        readonly DOCUMENT_POSITION_CONTAINED_BY: number;
        readonly DOCUMENT_POSITION_CONTAINS: number;
        readonly DOCUMENT_POSITION_DISCONNECTED: number;
        readonly DOCUMENT_POSITION_FOLLOWING: number;
        readonly DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC: number;
        readonly DOCUMENT_POSITION_PRECEDING: number;
        readonly DOCUMENT_TYPE_NODE: number;
        readonly ELEMENT_NODE: number;
        readonly ENTITY_NODE: number;
        readonly ENTITY_REFERENCE_NODE: number;
        readonly NOTATION_NODE: number;
        readonly PROCESSING_INSTRUCTION_NODE: number;
        readonly TEXT_NODE: number;
        dispatchEvent(event: Event): boolean;
        ariaAtomic: string;
        ariaAutoComplete: string;
        ariaBusy: string;
        ariaChecked: string;
        ariaColCount: string;
        ariaColIndex: string;
        ariaColIndexText: string;
        ariaColSpan: string;
        ariaCurrent: string;
        ariaDisabled: string;
        ariaExpanded: string;
        ariaHasPopup: string;
        ariaHidden: string;
        ariaInvalid: string;
        ariaKeyShortcuts: string;
        ariaLabel: string;
        ariaLevel: string;
        ariaLive: string;
        ariaModal: string;
        ariaMultiLine: string;
        ariaMultiSelectable: string;
        ariaOrientation: string;
        ariaPlaceholder: string;
        ariaPosInSet: string;
        ariaPressed: string;
        ariaReadOnly: string;
        ariaRequired: string;
        ariaRoleDescription: string;
        ariaRowCount: string;
        ariaRowIndex: string;
        ariaRowIndexText: string;
        ariaRowSpan: string;
        ariaSelected: string;
        ariaSetSize: string;
        ariaSort: string;
        ariaValueMax: string;
        ariaValueMin: string;
        ariaValueNow: string;
        ariaValueText: string;
        role: string;
        animate(keyframes: Keyframe[] | PropertyIndexedKeyframes, options?: number | KeyframeAnimationOptions): Animation;
        getAnimations(options?: GetAnimationsOptions): Animation[];
        after(...nodes: (string | Node)[]): void;
        before(...nodes: (string | Node)[]): void;
        remove(): void;
        replaceWith(...nodes: (string | Node)[]): void;
        innerHTML: string;
        readonly nextElementSibling: Element;
        readonly previousElementSibling: Element;
        readonly childElementCount: number;
        readonly children: HTMLCollection;
        readonly firstElementChild: Element;
        readonly lastElementChild: Element;
        append(...nodes: (string | Node)[]): void;
        prepend(...nodes: (string | Node)[]): void;
        querySelector<K_6 extends keyof HTMLElementTagNameMap>(selectors: K_6): HTMLElementTagNameMap[K_6];
        querySelector<K_7 extends keyof SVGElementTagNameMap>(selectors: K_7): SVGElementTagNameMap[K_7];
        querySelector<E_1 extends Element = Element>(selectors: string): E_1;
        querySelectorAll<K_8 extends keyof HTMLElementTagNameMap>(selectors: K_8): NodeListOf<HTMLElementTagNameMap[K_8]>;
        querySelectorAll<K_9 extends keyof SVGElementTagNameMap>(selectors: K_9): NodeListOf<SVGElementTagNameMap[K_9]>;
        querySelectorAll<E_2 extends Element = Element>(selectors: string): NodeListOf<E_2>;
        replaceChildren(...nodes: (string | Node)[]): void;
        readonly assignedSlot: HTMLSlotElement;
        oncopy: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
        oncut: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
        onpaste: (this: DocumentAndElementEventHandlers, ev: ClipboardEvent) => any;
        readonly style: CSSStyleDeclaration;
        contentEditable: string;
        enterKeyHint: string;
        inputMode: string;
        readonly isContentEditable: boolean;
        onabort: (this: GlobalEventHandlers, ev: UIEvent) => any;
        onanimationcancel: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
        onanimationend: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
        onanimationiteration: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
        onanimationstart: (this: GlobalEventHandlers, ev: AnimationEvent) => any;
        onauxclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onbeforeinput: (this: GlobalEventHandlers, ev: InputEvent) => any;
        onblur: (this: GlobalEventHandlers, ev: FocusEvent) => any;
        oncancel: (this: GlobalEventHandlers, ev: Event) => any;
        oncanplay: (this: GlobalEventHandlers, ev: Event) => any;
        oncanplaythrough: (this: GlobalEventHandlers, ev: Event) => any;
        onchange: (this: GlobalEventHandlers, ev: Event) => any;
        onclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onclose: (this: GlobalEventHandlers, ev: Event) => any;
        oncontextmenu: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        oncuechange: (this: GlobalEventHandlers, ev: Event) => any;
        ondblclick: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        ondrag: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondragend: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondragenter: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondragleave: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondragover: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondragstart: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondrop: (this: GlobalEventHandlers, ev: DragEvent) => any;
        ondurationchange: (this: GlobalEventHandlers, ev: Event) => any;
        onemptied: (this: GlobalEventHandlers, ev: Event) => any;
        onended: (this: GlobalEventHandlers, ev: Event) => any;
        onerror: OnErrorEventHandlerNonNull;
        onfocus: (this: GlobalEventHandlers, ev: FocusEvent) => any;
        onformdata: (this: GlobalEventHandlers, ev: FormDataEvent) => any;
        ongotpointercapture: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        oninput: (this: GlobalEventHandlers, ev: Event) => any;
        oninvalid: (this: GlobalEventHandlers, ev: Event) => any;
        onkeydown: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
        onkeypress: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
        onkeyup: (this: GlobalEventHandlers, ev: KeyboardEvent) => any;
        onload: (this: GlobalEventHandlers, ev: Event) => any;
        onloadeddata: (this: GlobalEventHandlers, ev: Event) => any;
        onloadedmetadata: (this: GlobalEventHandlers, ev: Event) => any;
        onloadstart: (this: GlobalEventHandlers, ev: Event) => any;
        onlostpointercapture: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onmousedown: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmouseenter: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmouseleave: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmousemove: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmouseout: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmouseover: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onmouseup: (this: GlobalEventHandlers, ev: MouseEvent) => any;
        onpause: (this: GlobalEventHandlers, ev: Event) => any;
        onplay: (this: GlobalEventHandlers, ev: Event) => any;
        onplaying: (this: GlobalEventHandlers, ev: Event) => any;
        onpointercancel: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerdown: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerenter: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerleave: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointermove: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerout: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerover: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onpointerup: (this: GlobalEventHandlers, ev: PointerEvent) => any;
        onprogress: (this: GlobalEventHandlers, ev: ProgressEvent<EventTarget>) => any;
        onratechange: (this: GlobalEventHandlers, ev: Event) => any;
        onreset: (this: GlobalEventHandlers, ev: Event) => any;
        onresize: (this: GlobalEventHandlers, ev: UIEvent) => any;
        onscroll: (this: GlobalEventHandlers, ev: Event) => any;
        onsecuritypolicyviolation: (this: GlobalEventHandlers, ev: SecurityPolicyViolationEvent) => any;
        onseeked: (this: GlobalEventHandlers, ev: Event) => any;
        onseeking: (this: GlobalEventHandlers, ev: Event) => any;
        onselect: (this: GlobalEventHandlers, ev: Event) => any;
        onselectionchange: (this: GlobalEventHandlers, ev: Event) => any;
        onselectstart: (this: GlobalEventHandlers, ev: Event) => any;
        onslotchange: (this: GlobalEventHandlers, ev: Event) => any;
        onstalled: (this: GlobalEventHandlers, ev: Event) => any;
        onsubmit: (this: GlobalEventHandlers, ev: SubmitEvent) => any;
        onsuspend: (this: GlobalEventHandlers, ev: Event) => any;
        ontimeupdate: (this: GlobalEventHandlers, ev: Event) => any;
        ontoggle: (this: GlobalEventHandlers, ev: Event) => any;
        ontouchcancel?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
        ontouchend?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
        ontouchmove?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
        ontouchstart?: (this: GlobalEventHandlers, ev: TouchEvent) => any;
        ontransitioncancel: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
        ontransitionend: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
        ontransitionrun: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
        ontransitionstart: (this: GlobalEventHandlers, ev: TransitionEvent) => any;
        onvolumechange: (this: GlobalEventHandlers, ev: Event) => any;
        onwaiting: (this: GlobalEventHandlers, ev: Event) => any;
        onwebkitanimationend: (this: GlobalEventHandlers, ev: Event) => any;
        onwebkitanimationiteration: (this: GlobalEventHandlers, ev: Event) => any;
        onwebkitanimationstart: (this: GlobalEventHandlers, ev: Event) => any;
        onwebkittransitionend: (this: GlobalEventHandlers, ev: Event) => any;
        onwheel: (this: GlobalEventHandlers, ev: WheelEvent) => any;
        autofocus: boolean;
        readonly dataset: DOMStringMap;
        nonce?: string;
        tabIndex: number;
        blur(): void;
        focus(options?: FocusOptions): void;
    };
} & TBase;

/**
 * A custom event that represents a `pending-state-error` event.
 * @public
 */
export declare type PendingStateErrorEvent = CustomEvent<PendingStateErrorEventData>;

/**
 * The data for a `pending-state-error` event.
 * @internal
 */
declare type PendingStateErrorEventData = {
    message?: string;
    error: Error;
};

/**
 * A custom event that represents a `pending-state` event.
 * @public
 */
export declare type PendingStateEvent = CustomEvent<PendingStateEventData>;

/**
 * The data for a `pending-state` event.
 * @internal
 */
declare type PendingStateEventData = {
    promise: Promise<void>;
};

/**
 * An enum that defines the names of the events that can be raised by the `PendingState` mixin.
 * @public
 */
export declare enum PendingStateEvents {
    state = "pending-state",
    error = "pending-state-error"
}

/**
 * The default height (in pixels) for pop-up windows.
 * @public
 */
export declare const POPUP_DEFAULT_HEIGHT = 600;

/**
 * The default width (in pixels) for pop-up windows.
 * @public
 */
export declare const POPUP_DEFAULT_WIDTH = 483;

/**
 * Publishes an event of a specific type to all subscribed listeners.
 * @public
 */
export declare type Publish<EventType> = (event: EventType) => void;

/**
 * Converts a plain object to a reactive, observable object.
 *
 * @privateRemarks
 * Code borrowed from fast-element v2. We can remove post upgrade.
 *
 * @example
 * An array or reactive items.
 * ```ts
 * this.todos = todosResponse.map((t) => reactive(t));
 * ```
 *
 * @example
 * Add item.
 * ```ts
 * add(description: string) {
 *    this.splice(this.todos.length, 0, reactive({ description, done: false }));
 * }
 * ```
 *
 * @example
 * Remove item.
 * ```ts
 * remove(todo: Todo) {
 *    const index = this.todos.indexOf(todo);
 *    index !== -1 && this.splice(index, 1);
 * }
 *```
 *
 * @param object - The object to make reactive.
 * @param deep - Indicates whether to deeply convert the object.
 * @returns The converted object.
 *
 * @beta
 */
export declare function reactive<T>(object: T, deep?: boolean): T;

/**
 * Defines a property changed handler that calls a render() method on the target as an internal observation enhancement.
 *
 * @remarks
 * This is useful if you have a lot of properties that all invalidate the internal state.
 * See {@link https://www.fast.design/docs/fast-element/observables-and-state#internal-observation} for more information.
 *
 * @param target - The target to define the property change handler on.
 * @param name - The property name.
 *
 * @example
 * ```ts
 *  @attr({ mode: 'boolean', attribute: 'line-numbers' })
 *  @renderOnChange
 *  lineNumbers: boolean = true;
 *
 *  @attr
 *  @renderOnChange
 *  indent: number | 'tab' = 2;
 *
 *  render() {
 *    if (!this.$fastController.isConnected) {
 *      return;
 *    }
 *    // Do something...
 *  }
 * ```
 *
 * @public
 */
export declare function renderOnChange(target: FASTElement & {
    render(): void;
}, name: string): void;

/**
 * Resolve a promise after a timeout.
 *
 * @example
 * ```ts
 * const result = await Promise.race([
 *  resourceChecks(sourceRef, resource),
 *  resolveAfter<MonitoredResourceChecksResult>(this.config.checkTimeout, () => ({
 *    isConnected: false,
 *    lastUpdated: Date.now(),
 *   })),
 * ]);
 * ```
 *
 * @public
 */
export declare const resolveAfter: <T = any>(ms: number, valueCreator: () => T) => Promise<T>;

/**
 * An object that defines two resource types: "local" and "remote".
 * @public
 */
export declare const ResourceType: {
    readonly local: "local";
    readonly remote: "remote";
};

/**
 * A type that represents the resource types defined by the `ResourceType` object.
 * @public
 */
export declare type ResourceType = (typeof ResourceType)[keyof typeof ResourceType];

/**
 * Setup an `IntersectionObserver` which will activate a callback function when an element becomes visible on screen
 * @param element - HTMLElement to observe
 * @param callback - any function called when the visibility changes
 * @public
 */
export declare const respondToVisibility: (element: HTMLElement, callback: (arg0: boolean) => any) => void;

/**
 * The data for a server row DTO.
 * @public
 */
export declare type ServerRowDTO = {
    DETAILS: {
        OPERATION: UppercaseOperation;
        ROW_REF: string;
    };
};

/**
 * A mapper for converting between server row DTOs and entities.
 * Provides methods to get/set a ServerRow DTO entity mapping.
 * @public
 */
export declare interface ServerRowDTOMapper extends DTOMapper<ServerRowDTO, ServerRowEntity> {
}

/**
 * A DI token used to obtain a `ServerRowDTOMapper` instance.
 * @public
 */
export declare const ServerRowDTOMapper: InterfaceSymbol<ServerRowDTOMapper>;

/**
 * The data for a server row entity.
 * @public
 */
export declare type ServerRowEntity = {
    serverRowMetadata: {
        operation: LowercaseOperation;
        rowRef: string;
    };
};

/**
 * A custom element that encapsulates a set of styles that can be applied to slotted elements.
 * @public
 */
export declare class SlottedStyles extends FASTElement {
    /**
     * The styles to apply to slotted elements.
     */
    styles: ElementStyles;
    /**
     * Called when the `styles` property changes.
     * Removes the previous styles from the parent element's FAST controller and adds the new styles.
     * @param prev - The previous value of the `styles` property.
     * @param next - The new value of the `styles` property.
     */
    stylesChanged(prev: ElementStyles, next: ElementStyles): void;
}

/**
 * The sub-path used for WebSocket connections when API_HOST is not set
 * @public
 * @defaultValue `'gwf'`
 * @remarks Optional.
 * @example
 * If `API_HOST` is not set, `SOCKET_EXT` is set to `/custom` and UI is served on `https://localhost:1234`
 * API host URL will be `wss://localhost:1234/custom`
 */
export declare let SOCKET_EXT: string;

/**
 * Subscribes a listener function to receive events of a specific type.
 * @returns An unsubscribe function.
 * @public
 */
export declare type Subscribe<EventType> = (listener: Listener<EventType>) => () => void;

/**
 * Creates a synchronization directive that binds a data source to an HTML element,
 * @public
 */
export declare function sync<TSource = any, TReturn = any>(binding: Binding<TSource, TReturn>, conversionType?: ConversionType, eventName?: EventName, keyAttr?: string): CaptureType<TSource>;

/**
 * toElementStyles.
 * @remarks
 * Converts {@link @microsoft/fast-element#ComposableStyles} or a ComposableStyles array to {@link @microsoft/fast-element#ElementStyles}.
 * @param styles - ComposableStyles or a ComposableStyles array.
 * @public
 */
export declare function toElementStyles(styles: ComposableStyles | ComposableStyles[]): ElementStyles;

/**
 * An object containing type ramp values.
 * @public
 */
export declare const TypeRampValues: {
    readonly minusOne: -1;
    readonly minusTwo: -2;
    readonly plusOne: 1;
    readonly plusTwo: 2;
    readonly plusThree: 3;
    readonly plusFour: 4;
    readonly plusFive: 5;
    readonly plusSix: 6;
};

/**
 * @internal
 */
declare type UppercaseOperation = Uppercase<LowercaseOperation>;

/**
 * An interface for generating UUIDs.
 * @public
 */
export declare interface UUID {
    /**
     * Generates a new UUID.
     * @param withLogging - Optional. Whether to log the generated UUID to the console. Defaults to false.
     * @returns The generated UUID.
     */
    createId(withLogging?: boolean): string;
    /**
     * Generates a new UUID for a remote object.
     * @param withLogging - Optional. Whether to log the generated UUID to the console. Defaults to false.
     * @returns A promise that resolves to the generated UUID.
     */
    createRemoteId(withLogging?: boolean): Promise<string>;
}

/**
 * A dependency injection token for the UUID interface.
 * @public
 */
export declare const UUID: InterfaceSymbol<UUID>;

/**
 * A configuration object for customizing UUID generation.
 * @public
 */
export declare interface UUIDConfig {
}

/**
 * Visit object utility.
 *
 * @privateRemarks
 * Code borrowed from fast-element v2. We can remove post upgrade.
 *
 * @example
 * Reactive example.
 * ```ts
 * export function reactive<T>(object: T, deep = false): T {
 *   visitObject(object, deep, makeObserverVisitor, void 0, observed);
 *   return object;
 * }
 * ```
 *
 * @param object - The object.
 * @param deep - A flag to indicate if a recursive visit of sub objects should occur.
 * @param visitor - The defined {@link ObjectVisitor} logic.
 * @param data - Visitor data.
 * @param traversed - The traversed object set.
 *
 * @beta
 */
export declare function visitObject<TVisitorData>(object: any, deep: boolean, visitor: ObjectVisitor<TVisitorData>, data: TVisitorData, traversed: WeakSet<any> | Set<any>): void;

/**
 * Directive that allows supplying an "else" template to the traditional {@link https://www.fast.design/docs/api/fast-element.when/#when-function} directive
 *
 * @param binding - The condition to test for rendering.
 * @param trueTemplateOrTemplateBinding - The template or a binding that gets the template to render when the condition is true.
 * @param falseTemplateOrTemplateBinding - The template or a binding that gets the template to render when the condition is false.
 * @public
 */
export declare function whenElse<TSource = any, TReturn = any>(binding: Binding<TSource, TReturn>, trueTemplateOrTemplateBinding: WhenTemplate<TSource>, falseTemplateOrTemplateBinding: WhenTemplate<TSource>): CaptureType<TSource>;

declare type WhenTemplate<TSource> = SyntheticViewTemplate | Binding<TSource, SyntheticViewTemplate>;

export { }
